/*
  Originally written by Naoto Kaneko
  Last updated: 28.Jun.2010
  Latest info: http://www37.atwiki.jp/pjkyt/
*/

package com.kyt.android.util;

import java.io.*;
import java.util.*;

import android.app.*;
import android.content.*;

public class FileDialogue implements DialogInterface.OnClickListener, DialogInterface.OnCancelListener, TextBoxDialogue.Listener {
	public interface Listener {
		// "" is given for the filename when the target is set DIRECTORY
		public void onSelected(String directory, String filename);
		public void onCanceled();
	}
	
	// target
	public static final int FILE = 0x0;
	public static final int DIRECTORY = 0x1;
	private int Target = FILE;
	
	// mode
	public static final int SELECT = 0x0;
	public static final int CREATE = 0x1;
	private int Mode = SELECT;
	
	private AlertDialog.Builder AdBuilder;

	private String InitDir = "/sdcard/";
	private ArrayList<String> Directory;
	private File[] FileList;
	private String[] FileNames;
	private String Title;
	
	private Context CallerContext;
	
	private FileDialogue.Listener FdListener;
	
	// initDir may be null or ""; the default InitDir will just be used
	// target is either FILE or DIRECTORY
	// context is usually that of caller Activity
	// fdListener is usually implemented by the caller class
	public FileDialogue(String initDir, int target, int mode, Context context, FileDialogue.Listener fdListener) {
		FdListener = fdListener;
		Target = target;
		Mode = mode;
		CallerContext = context;
		
		Title = makeTitle();
		Directory = initDirectory(initDir);
				
		AdBuilder = new AlertDialog.Builder(context);
	}
	
	private String makeTitle() {
		if(Target == FILE && Mode == SELECT) {
			return "Select File: ";
		}
		else if(Target == DIRECTORY && Mode == SELECT) {
			return "Select Directory: ";
		}
		if(Target == FILE && Mode == CREATE) {
			return "Create File: ";
		}
		else  {
			return "Create Directory: ";
		}	
	}
	
	// assumption is that containing direcotry names ends with /
	private String getPathString() {
		String path = "";
		
		for(int n=0; n!=Directory.size(); n++) {
			path += Directory.get(n);
		}
		return path;
	}
	
	private ArrayList<String> initDirectory(String initDir) {
		ArrayList<String> directory = new ArrayList<String>();
		
		if(initDir == null || initDir == "") {
			// init paht is not specified
			initDir = InitDir;
		}
		else {
			File test = new File(initDir);
			
			if(test.isDirectory() == false) {
				// specified init path is invalid
				initDir = InitDir;
			}
		}
		
		int str_len = initDir.length();
		int token_start = 0;
		
		for(int n=0; n!=str_len; n++) {
			
			if(initDir.charAt(n) == '/') {
				directory.add(initDir.substring(token_start,n+1));
				token_start = n+1;
			}
			else if(n == str_len-1) {
				// in case the given path string didn't end with '/'
				directory.add(initDir.substring(token_start,n+1)+"/");
			}
		}	
		
		return directory;
	}
		
	public void showDialogue() {
		FileList = makeFileList();		
		FileNames = makeFilenameList(FileList);
		
		AdBuilder.setTitle(Title + getPathString());
		AdBuilder.setItems(FileNames, this);
		AdBuilder.setOnCancelListener(this);
		
		AdBuilder.create().show();	
	}
	
	private class FilenameComparator implements Comparator<File> {
		public int compare(File file1, File file2) {
			if(file1.isDirectory() && file2.isFile()) {
				// directories line up before files
				return -1;
			}
			else if(file1.isFile() && file2.isDirectory()) {
				// directories line up before files
				return 1;
			}
			else {
				// simple string comparison
				return file1.getName().compareTo(file2.getName());
			}
		}
	}
	
	private File[] makeFileList() {
		File[] file_list = new File(getPathString()).listFiles();
		Arrays.sort(file_list, new FilenameComparator());
		
		return file_list;
	}
	
	private String[] makeFilenameList(File[] fileList) {
		int offset = getFileListOffset(); 
		
		String[] filename_list = new String[fileList.length+offset];
		filename_list[0] = "<Up>"; // == ".."
		
		if(offset == 2) {
			filename_list[1] = "<Here>"; // == "."
		}
		
		for(int n=0; n!=fileList.length; n++) {
			if(fileList[n].isDirectory() == true) {
				filename_list[n+offset] = fileList[n].getName() + "/";
			}
			else {
				filename_list[n+offset] = fileList[n].getName();
			}
		}	
		return filename_list;
	}
	
	private int getFileListOffset() {
		// (Target == FILE && Mode == SELECT) will put <UP> on top
		// Other conditions will put  <UP> and <HERE> on top 
		return (Target == FILE && Mode == SELECT) ? 1 : 2;	
	}
	
	// prompt for the filename / direcotry name entry using TextBoxDialogue
	private void showTextBoxDialogue(String filename) {
		String title;
		
		if(Target == FILE) {
			title = "New File Name";
		}
		else {
			title = "New Directory Name";
		}
		TextBoxDialogue tbDialogue =  new TextBoxDialogue(title, filename, CallerContext, this);
		tbDialogue.showDialogue();	
	}
	
	// AlertDialogListener ----
	
	public void onClick(DialogInterface dialog, int which) {
		int offset = getFileListOffset();
		
		if(which == 0) { // "<UP>" selected
			if(Directory.size() != 1) {
				// == 1 means, it is the root directory so can't go any higher
				Directory.remove(Directory.size()-1);
			}
			showDialogue();
		}
		else if((Target == DIRECTORY || Mode == CREATE) && which == 1) { // "<Here>" selected
			// direcotry selected
			
			if(Mode == CREATE) {
				// prompting for the new file name / direcotry name
				showTextBoxDialogue(null);
			}
			else {
				// Target == DIRECTORY && Mode == SELECT
				FdListener.onSelected(getPathString(), "");
			}
		}
		else if(FileList[which-offset].isDirectory() == true) {
			// if it is directory, going down the path
			Directory.add(FileNames[which]);
			showDialogue();
		}
		else if(Target == FILE) {
			// file selected
			if(Mode == CREATE) {
				// prompting for the new filename 
				//  with the selected file name filled in the text box
				showTextBoxDialogue(FileNames[which]);
			}
			else {
				FdListener.onSelected(getPathString(), FileNames[which]);
			}
		}
		else {
			// file selected despite the target is directory; asking again
			showDialogue();
		}
	}
	
 	public void onCancel(DialogInterface dialog) {
 		// User has pressed the BACK button
 		FdListener.onCanceled();
 	}
 	
	// TextBoxDialogueListener ----
 	
	public void onEntered(String text) {
		
		if(Target == FILE) {
			// User has entered a filename to be created
			FdListener.onSelected(getPathString(), text);
		}
		else {
			// User has entered a directory name to be created
			Directory.add(text);
			FdListener.onSelected(getPathString(), "");
		}
	}
	
	public void onCanceled() {
		// User has canceled the entry of the filename
		showDialogue();
	}

}
