/*
  Originally written by Naoto Kaneko
  Last updated: 28.Jun.2010
  Latest info: http://www37.atwiki.jp/pjkyt/
*/

package com.kyt.android.util;

import android.app.AlertDialog;
import android.content.*;
import android.widget.*;

public class TextBoxDialogue implements DialogInterface.OnClickListener, DialogInterface.OnCancelListener {
	public interface Listener {
		public void onEntered(String text);
		public void onCanceled();
	}
	
	private CharSequence Title;
	private EditText TextBox;
	private AlertDialog.Builder AdBuilder;
	
	private TextBoxDialogue.Listener TbdListener;
	
	public TextBoxDialogue(CharSequence title, CharSequence textBoxContent, Context context, TextBoxDialogue.Listener tbdListener) {
		Title = title;
		TbdListener = tbdListener;
		
		TextBox = new EditText(context);
		
		if(textBoxContent != null && textBoxContent != "") {
			TextBox.setText(textBoxContent);
		}
		AdBuilder = new AlertDialog.Builder(context);
	}
	
	public void showDialogue() {
		AdBuilder.setTitle(Title);
		AdBuilder.setView(TextBox);
		AdBuilder.setPositiveButton("Okay",this);
		AdBuilder.setNegativeButton("Cancel",this);
		AdBuilder.setOnCancelListener(this);
		
		AdBuilder.create().show();	
	}
	
    public void onClick(DialogInterface dialog, int which) {
        if(which == DialogInterface.BUTTON_POSITIVE){
        	TbdListener.onEntered(TextBox.getText().toString());
        }
        else if(which == DialogInterface.BUTTON_NEGATIVE) {
        	TbdListener.onCanceled();
        }
    }
    
 	public void onCancel(DialogInterface dialog) {
 		TbdListener.onCanceled();
 	}

}
